﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Diagnostics;

namespace PeteBrown.SilverlightBigWindow
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            DisplayManager.LoadDisplays();

            DisplayList.ItemsSource = DisplayManager.Displays;
        }

        private void OpenWindow_Click(object sender, RoutedEventArgs e)
        {
            if (DisplayManager.Displays.Count > 0)
            {
                Window w = new Window();

                w.Left = (from DisplayInfo di in DisplayManager.Displays
                          select di.MonitorArea.Left).Min();

                w.Top = (from DisplayInfo di in DisplayManager.Displays
                         select di.MonitorArea.Top).Min();

                w.Width = (from DisplayInfo di in DisplayManager.Displays
                           select di.Width).Sum();

                w.Height = (from DisplayInfo di in DisplayManager.Displays
                            select di.Height).Sum();

                MediaElement video = new MediaElement();
                video.Width = w.Width;
                video.Height = w.Height;
                video.Source = new Uri("http://10rem.net/pub/sl5iA/NetduinoRobot_SmallM.wmv", UriKind.Absolute);
                video.Stretch = Stretch.UniformToFill;
                video.AutoPlay = true;

                w.Content = video;

                w.Show();
            }
            else
            {
                MessageBox.Show("Display list not yet loaded.");
            }
        }


        private void PositionWindowOnSingleScreen(DisplayInfo screen, Window window, bool SizeToScreen = true, bool maximize = false)
        {
            window.Left = screen.MonitorArea.Left;
            window.Top = screen.MonitorArea.Top;

            if (SizeToScreen)
            {
                window.Width = screen.Width;
                window.Height = screen.Height;
            }

            if (maximize)
                window.WindowState = WindowState.Maximized;
        }


        private void OpenPrimary_Click(object sender, RoutedEventArgs e)
        {
            if (DisplayManager.Displays.Count > 0)
            {
                // find primary display
                DisplayInfo info = (from DisplayInfo di in DisplayManager.Displays
                                    where di.IsPrimary
                                    select di).FirstOrDefault();

                if (info != null)
                {
                    Window w = new Window();

                    PositionWindowOnSingleScreen(info, w, true, false);

                    w.Show();
                }
                else
                {
                    MessageBox.Show("No primary display. I suppose you won't see this message.");
                }
            }
            else
            {
                MessageBox.Show("Display list not yet loaded.");
            }
        }

        private void OpenSecondary_Click(object sender, RoutedEventArgs e)
        {
            if (DisplayManager.Displays.Count > 0)
            {
                // find first secondary display
                DisplayInfo info = (from DisplayInfo di in DisplayManager.Displays
                                    where !di.IsPrimary
                                    select di).FirstOrDefault();

                if (info != null)
                {
                    Window w = new Window();

                    PositionWindowOnSingleScreen(info, w, true, false);

                    w.Show();
                }
                else
                {
                    MessageBox.Show("No secondary display available.");
                }
            }
            else
            {
                MessageBox.Show("Display list not yet loaded.");
            }
        }


        private void MoveMainWindowToSelectedDisplay()
        {
            // see comments in blog post as to why I went with strings
            string deviceName = ((DisplayInfo)DisplayList.SelectedItem).MonitorName;

            DisplayInfo display = (from DisplayInfo di in DisplayManager.Displays
                                    where di.MonitorName.ToLower() == deviceName.ToLower()
                                    select di).FirstOrDefault();

            var mainWindow = Application.Current.MainWindow;

            // center on the display

            mainWindow.Left = display.MonitorArea.Left + (display.Width - mainWindow.Width) / 2;
            mainWindow.Top = display.MonitorArea.Top + (display.Height - mainWindow.Height) / 2;

        }

        private void MoveToSelectedButton_Click(object sender, RoutedEventArgs e)
        {
            if (DisplayList.SelectedItem != null)
            {
                MoveMainWindowToSelectedDisplay();
            }
            else
            {
                MessageBox.Show("Please select a screen from the list");
            }
        }
    }
}
